#!/usr/bin/python3
#
# -*- coding: utf-8 -*-
#
# Univention UCS@school
# Copyright 2020-2022 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

import csv
import random
import shutil
import datetime
import univention.admin.uldap

from ucsschool.importer.utils.pre_read_pyhook import PreReadPyHook


class GenerateRecordUID(PreReadPyHook):
    """
    Hook is called before starting to read the input file to add randomly
    generated record_uid to the CSV input file.
    """

    name = "generate_record_uid"
    priority = {
        "pre_read": 1000,
    }

    lo, po = univention.admin.uldap.getMachineConnection()

    def get_random_unique_record_uid(self):
        already_used = True
        while already_used:
            new_record_uid = "s{}".format(random.randint(1, 99999999))
            obj = self.lo.search("ucsschoolRecordUID={}".format(new_record_uid))
            if obj:
                self.logger.warn(
                    "Randomly generated record_uid already used: {}! Generating new one...".format(
                        new_record_uid
                    )
                )
            else:
                already_used = False
            return new_record_uid

    # get delimiter, but only read the 1st 1024 bytes to save time with big CSV files
    def get_delimiter(self, file_path):  # type: () -> str
        sniffer = csv.Sniffer()
        with open(file_path) as csvfile:
            dialect = csv.Sniffer().sniff(csvfile.read(1024))
        return dialect.delimiter

    def pre_read(self):  # type: () -> None
        """
        * backup original CSV file to {input:filename}.$date.bak.csv
        * rewrite original CSV file with randomly generated record_uid

        :return: None
        """
        # Dont do anything if the current import is not for students
        if self.config["user_role"] != "student":
            self.logger.info(
                "{}: user_role in config is not student. Not doing anything.".format(
                    self.name
                )
            )
            return

        ori_file_name = self.config["input"]["filename"]

        # get delimiter of original file
        delimiter = self.get_delimiter(ori_file_name)
        self.logger.info("{}: detected delimiter as {}".format(self.name, delimiter))

        # make backup of original file
        _fn = ori_file_name[:-4] if ori_file_name.endswith(".csv") else ori_file_name
        backup_file_name = "{}.ori.{:%Y-%m-%d_%H:%M:%S}.csv".format(
            _fn, datetime.datetime.now()
        )
        self.logger.info(
            "{}: Copying {} to {}...".format(self.name, ori_file_name, backup_file_name)
        )
        shutil.copy2(ori_file_name, backup_file_name)

        # rewrite CSV with added record_uid
        self.logger.info("Adding record_uids to {}...".format(ori_file_name))
        with open(backup_file_name, "r") as fpr, open(ori_file_name, "w") as fpw:
            reader = csv.DictReader(fpr, delimiter=delimiter)
            writer = csv.writer(fpw, delimiter=delimiter)
            if "record_uid" not in reader.fieldnames:
                reader.fieldnames.append("record_uid")
            writer.writerow(reader.fieldnames)
            for row in reader:
                self.logger.info("{}: Old row:      {}".format(self.name, row))
                if not row["record_uid"]:
                    row["record_uid"] = self.get_random_unique_record_uid()
                self.logger.info("{}: New row:      {}".format(self.name, row))
                writer.writerow([(row[str(fieldname)]) for fieldname in reader.fieldnames])
        self.logger.info("{}: Done rewriting CSV file.".format(self.name))
