#!/usr/bin/python3
#
# -*- coding: utf-8 -*-
#
# Univention UCS@school
# Copyright 2020-2022 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

"""
 1) PyHook to mail new user passwords
"""

import shutil
import os

from ucsschool.importer.utils.result_pyhook import ResultPyHook
from ucsschool.importer.configuration import Configuration
from ucsschool.importer.exceptions import (
    InitialisationError,
    MissingSchoolName,
    UcsSchoolImportError,
)

try:
    from ucsschool.lib.models.utils import exec_cmd
except ImportError:
    raise InitialisationError(
        "The 'send_passwords' hook needs to be executed on UCS@school >= 4.4 v3."
    )


class MailResultHook(ResultPyHook):
    name = "send_passwords"
    priority = {
        "user_result": 1,
    }
    supports_dry_run = True

    def user_result(self, user_import_data):
        config = Configuration()
        self.logger.info("{}: Starting".format(self.name))
        if self.dry_run:
            self.logger.info("{}: Dry-Run, not doing anything".format(self.name))
            return
        school = config["school"]
        if not school:
            raise MissingSchoolName(
                "Hook 'send_import_passwords' requires a fixed value for 'school' in "
                "the configuration. Works, when run by the import user UMC module."
            )
        new_user_passwords_path = config["output"]["new_user_passwords"]
        if not new_user_passwords_path or not os.path.exists(new_user_passwords_path):
            raise UcsSchoolImportError(
                "Hook 'send_import_passwords' requires the configuration value of "
                "'output->new_user_passwords' to point diretly to a file. Works, "
                "when run by the import user UMC module."
            )
        self.logger.info(
            "{}: Calling 'send_import_passwords' with school {!r} and CSV file "
            "{!r}...".format(self.name, school, new_user_passwords_path)
        )
        exec_cmd(
            ["/usr/sbin/send_import_passwords", "-s", school, new_user_passwords_path],
            log=True,
            raise_exc=True,
        )
        moved_user_passwords_path = os.path.join(
            os.path.dirname(new_user_passwords_path),
            "{}.moved".format(os.path.basename(new_user_passwords_path)),
        )
        shutil.move(new_user_passwords_path, moved_user_passwords_path)
        self.logger.info(
            "Moved password-file from {!r} to {!r}".format(
                new_user_passwords_path, moved_user_passwords_path
            )
        )


# vim: set ts=4 sw=4 et
